@@ -144,21 +144,20 @@ class CyberGISCompute:
         """
         Checks for json file and calls login_token function.
         """
-        with open(os.path.abspath('cybergis_compute_user.json')) as f:
-            user = json.load(f)
-            token = user['token']
-            print('📃 Found "cybergis_compute_user.json"')
-            print('NOTE: if you want to login as another user, please remove this file')
-            try:
-                self.jupyterhubApiToken = token
-                self.set_username()
-                self.save_token()
-                return self.login()
-            except:
-                envToken = os.getenv('JUPYTERHUB_API_TOKEN')
-                if envToken is not None:
-                    return self.host_token_login(envToken)
-                print('❌ Failed to login via token JSON file')
+        try:
+            with open(os.path.abspath('cybergis_compute_user.json')) as f:
+                user = json.load(f)
+                token = user['token']
+            print('📃 Found "cybergis_compute_user.json! NOTE: if you want to login as another user, please remove this file')
+            self.jupyterhubApiToken = token
+            self.set_username()
+            self.save_token()
+            return self.login()
+        except:
+            # print('❌ Failed to login via token JSON file, trying environment variable...')
+            envToken = os.getenv('JUPYTERHUB_API_TOKEN')
+            if envToken is not None:
+                return self.host_token_login(envToken)
 
     def login(self, manualLogin=False, manualHost=None, verbose=True):
         """
@@ -171,7 +170,6 @@ class CyberGISCompute:
         Todo:
             Document exceptions/errors raised.
         """
-
         if manualHost is not None:
             self.jupyterhubHost = manualHost
         # login via env variable
@@ -270,23 +268,22 @@ class CyberGISCompute:
                    'remoteResultFolder', 'param', 'slurm', 'userId', 'maintainer', 'createdAt']
         data = []
         for job in jobs['job']:
-            data.append([
+            to_append = [
                 job['id'],
                 job['hpc'],
                 job['remoteExecutableFolder']["id"] if (
-                    "id" in job['remoteExecutableFolder']) else None,
+                    job['remoteExecutableFolder'] is not None and "id" in job['remoteExecutableFolder']) else None,
                 job['remoteDataFolder']["id"] if (
-                    "id" in job['remoteDataFolder']) else None,
+                    job['remoteDataFolder'] is not None and "id" in job['remoteDataFolder']) else None,
                 job['remoteResultFolder']["id"] if (
-                    "id" in job['remoteResultFolder']) else None,
-                job['remoteDataFolder'],
-                job['remoteResultFolder'],
+                    job['remoteResultFolder'] is not None and "id" in job['remoteResultFolder']) else None,
                 json.dumps(job['param']),
                 json.dumps(job['slurm']),
                 job['userId'],
                 job['maintainer'],
-                job['createdAt'],
-            ])
+                job['createdAt']
+            ]
+            data.append(to_append)
 
         if self.isJupyter:
             if len(data) == 0:
@@ -363,6 +360,41 @@ class CyberGISCompute:
         else:
             print(MarkdownTable.render(data, headers))
 
+    def list_jupyter_host(self, raw=False):
+        """
+        Prints a list of jupyter hosts that the server supports
+
+        Args:
+            raw (bool): set to True if you want the raw output
+
+        Returns:
+            JSON: Raw output if raw=True otherwise its
+            printed or displayed directly into the interface
+        """
+        try:
+            hosts = self.client.request('GET', '/whitelist')['whitelist']
+            if raw:
+                return hosts
+
+            headers = ['jupyter_host', 'description']
+            data = []
+
+            for i in hosts:
+                data.append([
+                    i,
+                    hosts[i],
+                ])
+
+            if self.isJupyter:
+                if len(data) == 0:
+                    print('empty')
+                    return
+                display(Markdown(MarkdownTable.render(data, headers)))
+            else:
+                print(MarkdownTable.render(data, headers))
+        except:
+            print("The server " + self.client.url + " doesn't have this route")
+
     def list_git(self, raw=False):
         """
         Prints a list of Git projects that the server supports
